from django import template
from django.core.urlresolvers import reverse

register = template.Library()

@register.tag
def masquerade_link(parser, token):
    return MasqueradeLinkNode()

class MasqueradeLinkNode(template.Node):
    def __init__(self):
        self.request = template.Variable('request')
    
    def render(self, context):
        request = self.request.resolve(context)

        link = '<a href="%s">%s</a>'

        try:
            if request.user.is_masked:
                link = link % (reverse('masquerade.views.unmask'),
                  'Unmask!')
            else:
                link = link % (reverse('masquerade.views.mask'),
                  'Masquerade as user')

        except AttributeError:
            return ''

        return link
        
@register.tag
def masquerade_status(parser, token):
    return MasqueradeStatusNode()

class MasqueradeStatusNode(template.Node):
    def __init__(self):
        self.request = template.Variable('request')
    
    def render(self, context):
        request = self.request.resolve(context)

        status = "You are not currently masquerading as any other user."

        try:
            if request.user.is_masked:
                status = "You are masquerading as %s (%s %s)" % (
                  request.user.username,
                  request.user.first_name,
                  request.user.last_name,
                )

        except AttributeError:
            pass

        return status




from django.conf import settings
try:
    from django.contrib.auth import get_user_model
    User = get_user_model()
    from django.contrib.auth.models import Group
except ImportError:
    from django.contrib.auth.models import User, Group

from django.core.urlresolvers import reverse
from django.http import HttpRequest
from django.test import TestCase, Client
from mock import Mock
from masquerade.middleware import MasqueradeMiddleware
import masquerade.views
import masquerade.signals

class MasqueradeTestCase(TestCase):
    """
    Tests for django-masquerade
    """

    def setUp(self):
        self.mask_on_signal_received = None
        self.mask_off_signal_received = None

        u = User.objects.create_user(username='generic',
          email='generic@foo.com', password='abc123')
        u.is_staff = False
        u.is_superuser = False
        u.save()

        u = User.objects.create_user(username='super',
          email='super@foo.com', password='abc123')
        u.is_staff = True
        u.is_superuser = True
        u.save()

        g = Group.objects.create(name='test_group')

        u = User.objects.create_user(username='staff',
          email='staff@foo.com', password='abc123')
        u.is_staff = True
        u.is_superuser = False
        u.save()
        u.groups.add(g)

        u = User.objects.create_user(username='group_member',
          email='group_member@foo.com', password='abc123')
        u.is_staff = True
        u.is_superuser = False
        u.save()
        u.groups.add(g)

    def test_mask_form_permissions(self):

        settings.MASQUERADE_REQUIRE_SUPERUSER = False
        settings.MASQUERADE_REQUIRE_COMMON_GROUP = False

        # log in as superuser
        c = Client()
        self.assertTrue(c.login(username='super', password='abc123'))

        # hit masquerade form with bad username
        response = c.post(reverse('masquerade-mask'), {'mask_user': 'nobody'})

        # verify form comes back with error
        self.assertTrue(response.status_code == 200)
        self.assertTrue(response.context['form'].is_valid() == False)

        # hit masquerade form with generic username
        response = c.post(reverse('masquerade-mask'),
          {'mask_user': 'generic'})
        self.assertTrue(response.status_code == 302)

        # make sure non-staff user cannot user form
        c = Client()
        c.login(username='generic', password='abc123')
        response = c.post(reverse('masquerade-mask'), {'mask_user': 'nobody'})
        self.assertTrue(response.status_code == 403)

        # make sure staff user can use form
        c = Client()
        c.login(username='staff', password='abc123')
        response = c.post(reverse('masquerade-mask'), {'mask_user': 'nobody'})
        self.assertTrue(response.status_code == 200)

        # ... unless require superuser setting is true.
        masquerade.views.MASQUERADE_REQUIRE_SUPERUSER = True

        c = Client()
        c.login(username='staff', password='abc123')
        response = c.post(reverse('masquerade-mask'), {'mask_user': 'nobody'})
        self.assertTrue(response.status_code == 403)

        masquerade.views.MASQUERADE_REQUIRE_SUPERUSER = False

        # hit masquerade with user of same group,
        # with REQUIRE_COMMON_GROUP as False
        c = Client()
        c.login(username='staff', password='abc123')
        response = c.post(reverse('masquerade-mask'),
          {'mask_user': 'group_member'})
        self.assertTrue(response.status_code == 302)

        # hit masquerade with user with not the same group,
        # with REQUIRE_COMMON_GROUP as False
        c = Client()
        c.login(username='staff', password='abc123')
        response = c.post(reverse('masquerade-mask'),
          {'mask_user': 'generic'})
        self.assertTrue(response.status_code == 302)

        masquerade.views.MASQUERADE_REQUIRE_COMMON_GROUP = True

        # hit masquerade with same group,
        # with REQUIRE_COMMON_GROUP as True
        c = Client()
        c.login(username='staff', password='abc123')
        response = c.post(reverse('masquerade-mask'),
          {'mask_user': 'group_member'})
        self.assertTrue(response.status_code == 302)

        # hit masquerade with user with not the same group,
        # with REQUIRE_COMMON_GROUP as True
        c = Client()
        c.login(username='staff', password='abc123')
        response = c.post(reverse('masquerade-mask'),
          {'mask_user': 'generic'})
        self.assertTrue(response.status_code == 200)
        self.assertTrue(response.context['form'].is_valid() == False)

    def test_mask_email(self):

        # log in as superuser
        c = Client()
        self.assertTrue(c.login(username='super', password='abc123'))

        # hit masquerade form with generic email
        response = c.post(reverse('masquerade-mask'),
          {'mask_user': 'generic@foo.com'})
        self.assertTrue(response.status_code == 302)

    def test_mask(self):
        mw = MasqueradeMiddleware()

        request = Mock(spec=HttpRequest)
        request.session = {'mask_user': 'generic'}
        request.user = User.objects.get_by_natural_key('super')

        mw.process_request(request)

        self.assertTrue(request.user.is_masked == True)
        self.assertTrue(request.user == User.objects.get_by_natural_key('generic'))
        self.assertTrue(request.user.original_user == User.objects.get_by_natural_key('super'))

    def test_unmask(self):
        mw = MasqueradeMiddleware()

        request = Mock(spec=HttpRequest)
        request.session = {}
        request.user = User.objects.get_by_natural_key('super')

        mw.process_request(request)

        self.assertTrue(request.user.is_masked == False)
        self.assertTrue(request.user == User.objects.get_by_natural_key('super'))
        self.assertTrue(request.user.original_user == None)

    def test_mask_on_signal_sent(self):
        def receiver(sender, mask_username, **kwargs):
            self.mask_on_signal_received = mask_username

        masquerade.signals.mask_on.connect(receiver)
        c = Client()
        c.login(username='super', password='abc123')
        c.post(reverse('masquerade-mask'),
          {'mask_user': 'generic'})
        self.assertEqual(self.mask_on_signal_received, 'generic')

    def test_mask_off_signal_sent(self):
        def receiver(sender, mask_username, **kwargs):
            self.mask_off_signal_received = mask_username

        masquerade.signals.mask_off.connect(receiver)
        c = Client()
        c.login(username='super', password='abc123')
        session = c.session
        session['mask_user'] = 'generic'
        session.save()
        c.get(reverse('masquerade-unmask'))
        self.assertEqual(self.mask_off_signal_received, 'generic')



from masquerade.users import UserModel

try:
    from django.utils.deprecation import MiddlewareMixin
except ImportError:
    class MiddlewareMixin(object):
        pass

class MasqueradeMiddleware(MiddlewareMixin):
    def process_request(self, request):
        """Checks for the presence of "mask_user" in the session. The value
        should be the username of the user to be masqueraded as. Note we
        also set the "is_masked" attribute to true in that case so that when we
        hit the "mask" or "unmask" URLs this user is properly recognized,
        rather than the user they're masquerading as :) """

        request.user.is_masked = False
        request.user.original_user = None

        if 'mask_user' in request.session:
            User = UserModel()
            try:
                original_user = request.user
                request.user = User.objects.get_by_natural_key(
                    request.session['mask_user']
                )
                request.user.is_masked = True
                request.user.original_user = original_user
            except User.DoesNotExist:
                pass



import sys

from django import forms
from django.conf import settings
from django.db.models import Q
from operator import __or__ as OR

try:
    from django.contrib.auth import get_user_model
    User = get_user_model()
except ImportError:
    from django.contrib.auth.models import User

# Monkey patch for compatibility with Python 3
PY3 = sys.version_info[0] == 3
if PY3:
    import functools
    reduce = functools.reduce


class MaskForm(forms.Form):
    mask_user = forms.CharField(max_length=254, label="Username or Email")

    def clean_mask_user(self):
        query = self.cleaned_data['mask_user']

        search = [Q(**{f: query}) for f in self.get_user_search_fields()]

        try:
            u = User.objects.get(reduce(OR, search))
            self.user = u
        except User.DoesNotExist:
            raise forms.ValidationError("User not found.")
        return u.get_username()

    def get_user_search_fields(self):
        return getattr(settings, 'MASQUERADE_USER_SEARCH_FIELDS',
                ['username', 'email'])



from django.conf.urls import url

from masquerade.views import mask, unmask

urlpatterns = [
    url(r'^mask/$', mask, name='masquerade-mask'),
    url(r'^unmask/$', unmask, name='masquerade-unmask'),
]



from django.conf import settings


def UserModel():
    try:
        from django.contrib.auth import get_user_model
        return get_user_model()
    except ImportError:
        from django.contrib.auth.models import User
        return User




import os
from distutils.core import setup

def read_file(filename):
    """Read a file into a string"""
    path = os.path.abspath(os.path.dirname(__file__))
    filepath = os.path.join(path, filename)
    try:
        return open(filepath).read()
    except IOError:
        return ''

# Use the docstring of the __init__ file to be the description
DESC = " ".join(__import__('masquerade').__doc__.splitlines()).strip()

setup(
    name = "django-masquerade",
    version = __import__('masquerade').get_version().replace(' ', '-'),
    url = 'https://bitbucket.org/technivore/django-masquerade',
    author = 'Matthew Rich',
    author_email = 'matthew@technivore.org',
    description = DESC,
    long_description = read_file('README'),
    packages = ['masquerade','masquerade.templatetags'],
    package_data = {'masquerade': ['templates/masquerade/*.html',]},
    classifiers = [
        'License :: OSI Approved :: Apache Software License',
        'Framework :: Django',
    ],
)



from django.conf import settings
from django.core.urlresolvers import reverse
from django.http import HttpResponseRedirect, HttpResponseForbidden, Http404
from django.shortcuts import render
from django.template import RequestContext
from django import forms
from masquerade.forms import MaskForm
from masquerade.signals import mask_on, mask_off

try:
    from django.forms.utils import ErrorList
except ImportError:
    from django.forms.util import ErrorList # compatibility with < Django 1.7

MASQUERADE_REDIRECT_URL = getattr(settings, 'MASQUERADE_REDIRECT_URL', '/')

MASQUERADE_REQUIRE_SUPERUSER = getattr(settings,
  'MASQUERADE_REQUIRE_SUPERUSER', False)

MASQUERADE_REQUIRE_COMMON_GROUP = getattr(settings,
  'MASQUERADE_REQUIRE_COMMON_GROUP', False)

def mask(request, template_name='masquerade/mask_form.html'):
    if not request.user.is_masked and not request.user.is_staff:
        return HttpResponseForbidden()
    elif not request.user.is_superuser and MASQUERADE_REQUIRE_SUPERUSER:
        return HttpResponseForbidden()

    if request.method == 'POST':
        form = MaskForm(request.POST)
        form.full_clean()

        if MASQUERADE_REQUIRE_COMMON_GROUP and form.is_valid():
            user_groups = request.user.groups.all()
            mask_groups = form.user.groups.all()

            # If the user is not super, and there are no common groups,
            # then deny access.
            if (
                not request.user.is_superuser and
                not any(x in mask_groups for x in user_groups)
            ):
                form._errors[forms.forms.NON_FIELD_ERRORS] = (
                    ErrorList([u"You may not access that username"])
                )

        if form.is_valid():
            # turn on masquerading
            request.session['mask_user'] = form.cleaned_data['mask_user']
            mask_on.send(sender=form,
                mask_username=form.cleaned_data['mask_user'])
            return HttpResponseRedirect(MASQUERADE_REDIRECT_URL)
    else:
        form = MaskForm()

    return render(request, template_name, {'form': form})

def unmask(request):
    # Turn off masquerading. Don't bother checking permissions.
    try:
        mask_username = request.session['mask_user']
        del(request.session['mask_user'])
        mask_off.send(sender=object(), mask_username=mask_username)
    except KeyError:
        pass

    return HttpResponseRedirect(MASQUERADE_REDIRECT_URL)
